import { S3ApiClient } from '../../common/index';
import { delay } from '../../common/delay';
import { safeCancel } from '../../common/cancel';
export class SymbolsApiClient {
    _client;
    uploadUrl = '/symsrv/uploadUrl';
    uploadCompleteUrl = '/symsrv/uploadComplete';
    _s3ApiClient = new S3ApiClient();
    _timer = delay;
    constructor(_client) {
        this._client = _client;
    }
    async postSymbols(database, application, version, files) {
        const promises = files
            .map(async (file) => {
            const originalStream = file.file;
            const [checkStream, uploadStream] = originalStream.tee();
            const reader = checkStream.getReader();
            try {
                const { value, done } = await reader.read();
                if (done) {
                    throw new Error('Could not read symbol file stream');
                }
                if (!this.isGzipMagicBytes(value)) {
                    throw new Error('Symbol file stream is not a gzipped stream');
                }
            }
            finally {
                reader.releaseLock();
                safeCancel(checkStream);
            }
            let uploadResponse;
            try {
                file.file = uploadStream;
                const presignedUrl = await this.getPresignedUrl(database, application, version, file);
                const additionalHeaders = {
                    'content-encoding': 'gzip'
                };
                uploadResponse = await this._s3ApiClient.uploadFileToPresignedUrl(presignedUrl, file, additionalHeaders);
                await this.postUploadComplete(database, application, version, file);
            }
            finally {
                safeCancel(uploadStream);
            }
            await this._timer(1000);
            return uploadResponse;
        });
        return Promise.all(promises);
    }
    async getPresignedUrl(database, appName, appVersion, file) {
        const formData = this._client.createFormData();
        formData.append('database', database);
        formData.append('appName', appName);
        formData.append('appVersion', appVersion);
        formData.append('size', `${file.uncompressedSize}`);
        formData.append('symFileName', file.name);
        formData.append('moduleName', file.moduleName);
        formData.append('dbgId', file.dbgId);
        formData.append('lastModified', `${file.lastModified?.toISOString()}`);
        formData.append('SendPdbsVersion', 'spdbsv2');
        const request = {
            method: 'POST',
            body: formData,
            cache: 'no-cache',
            credentials: 'include',
            redirect: 'follow',
            duplex: 'half'
        };
        const response = await this._client.fetch(this.uploadUrl, request);
        if (response.status === 429) {
            throw new Error('Error getting presigned URL, too many requests');
        }
        if (response.status === 403) {
            throw new Error('Error getting presigned URL, invalid credentials');
        }
        if (response.status !== 200) {
            throw new Error(`Error getting presigned URL for ${file.name}`);
        }
        const json = await response.json();
        if (json.Status === 'Failed') {
            throw new Error(json.Error);
        }
        return json.url;
    }
    async postUploadComplete(database, appName, appVersion, file) {
        const formData = this._client.createFormData();
        formData.append('database', database);
        formData.append('appName', appName);
        formData.append('appVersion', appVersion);
        formData.append('size', `${file.uncompressedSize}`);
        formData.append('symFileName', file.name);
        formData.append('moduleName', file.moduleName);
        formData.append('dbgId', file.dbgId);
        formData.append('lastModified', `${file.lastModified}`);
        formData.append('SendPdbsVersion', 'spdbsv2');
        const request = {
            method: 'POST',
            body: formData,
            cache: 'no-cache',
            credentials: 'include',
            redirect: 'follow',
            duplex: 'half'
        };
        const response = await this._client.fetch(this.uploadCompleteUrl, request);
        if (response.status !== 200) {
            throw new Error(`Error completing symbol upload for ${file.name}, status ${response.status}`);
        }
        const json = await response.json();
        if (json.Status === 'Failed') {
            throw new Error(json.Error);
        }
        return response;
    }
    isGzipMagicBytes(buffer) {
        const view = new Uint8Array(buffer);
        return view[0] === 0x1f && view[1] === 0x8b;
    }
}
//# sourceMappingURL=symbols-api-client.js.map