import { S3ApiClient, TableDataClient } from '../../common/index';
import { delay } from '../../common/delay';
import { VersionsApiRow } from '../versions-api-row/versions-api-row';
export class VersionsApiClient {
    _client;
    route = '/api/versions';
    _s3ApiClient = new S3ApiClient();
    _tableDataClient;
    _timer = delay;
    constructor(_client) {
        this._client = _client;
        this._tableDataClient = new TableDataClient(this._client, this.route);
    }
    async getVersions(request) {
        const response = await this._tableDataClient.getData(request);
        const json = await response.json();
        const pageData = json.pageData;
        const rows = json.rows.map(row => new VersionsApiRow(row));
        return {
            rows,
            pageData
        };
    }
    async deleteVersions(database, appVersions) {
        const appVersionsParam = appVersions.reduce((prev, curr) => [...prev, curr.application, curr.version], []).join(',');
        const route = `${this.route}?database=${database}&appVersions=${appVersionsParam}`;
        const request = {
            method: 'DELETE',
            cache: 'no-cache',
            credentials: 'include',
            redirect: 'follow'
        };
        const response = await this._client.fetch(route, request);
        if (response.status !== 200) {
            throw new Error(`Error deleting symbols for ${database}-${appVersionsParam} status ${response.status}`);
        }
        const json = await response.json();
        if (json.Status === 'Failed') {
            throw new Error(json.Error);
        }
        return response;
    }
    async putFullDumps(database, application, version, fullDumps) {
        const fullDumpsFlag = fullDumps ? '1' : '0';
        const route = `${this.route}?database=${database}&appName=${application}&appVersion=${version}&fullDumps=${fullDumpsFlag}`;
        const request = {
            method: 'PUT',
            cache: 'no-cache',
            credentials: 'include',
            redirect: 'follow'
        };
        return this._client.fetch(route, request);
    }
    async putRetired(database, application, version, retired) {
        const retiredFlag = retired ? '1' : '0';
        const route = `${this.route}?database=${database}&appName=${application}&appVersion=${version}&retired=${retiredFlag}`;
        const request = {
            method: 'PUT',
            cache: 'no-cache',
            credentials: 'include',
            redirect: 'follow'
        };
        return this._client.fetch(route, request);
    }
    async deleteSymbols(database, application, version) {
        const route = `${this.route}?database=${database}&appName=${application}&appVersion=${version}`;
        const request = {
            method: 'DELETE',
            cache: 'no-cache',
            credentials: 'include',
            redirect: 'follow'
        };
        const response = await this._client.fetch(route, request);
        if (response.status !== 200) {
            throw new Error(`Error deleting symbols for ${database}-${application}-${version} status ${response.status}`);
        }
        const json = await response.json();
        if (json.Status === 'Failed') {
            throw new Error(json.Error);
        }
        return response;
    }
    async postSymbols(database, application, version, files) {
        const promises = files
            .map(async (file) => {
            const presignedUrl = await this.getPresignedUrl(database, application, version, file);
            const response = await this._s3ApiClient.uploadFileToPresignedUrl(presignedUrl, file);
            await this._timer(1000);
            return response;
        });
        return Promise.all(promises);
    }
    async getPresignedUrl(database, appName, appVersion, file) {
        const formData = this._client.createFormData();
        formData.append('database', database);
        formData.append('appName', appName);
        formData.append('appVersion', appVersion);
        formData.append('size', `${file.size}`);
        formData.append('symFileName', file.name);
        if (file.dbgId) {
            formData.append('dbgId', file.dbgId);
        }
        if (file.lastModified) {
            formData.append('lastModified', `${file.lastModified}`);
        }
        if (file.moduleName) {
            formData.append('moduleName', file.moduleName);
        }
        const request = {
            method: 'POST',
            body: formData,
            cache: 'no-cache',
            credentials: 'include',
            redirect: 'follow',
            duplex: 'half'
        };
        const response = await this._client.fetch(this.route, request);
        if (response.status === 429) {
            throw new Error('Error getting presigned URL, too many requests');
        }
        if (response.status === 403) {
            throw new Error('Error getting presigned URL, invalid credentials');
        }
        if (response.status !== 200) {
            throw new Error(`Error getting presigned URL for ${file.name}`);
        }
        const json = await response.json();
        if (json.Status === 'Failed') {
            throw new Error(json.Error);
        }
        return json.url;
    }
}
//# sourceMappingURL=versions-api-client.js.map